/*************************** CLASSES.H ************************************/

class CPlayer;

class CObject
{
  protected: int anim;                  // Animationstatus
	     int kpos;
	     int jump_power;
	     int speed;

 public:
	     unsigned int score;
	     int timer;
	     int fall;
	     char identity;             // 0 ... Player
					// 1 ... Coin
					// 2 ... Turtle
					// 3 ... Roboter
					// 4 ... Humer
					// 5 ... Wasser
					// 6 ... Sturm
					// 7 ... POW

	     CObject *prev,*next;       // fr doppelt verkettete Liste

	     int stimer;
	     int x, y;                  // Position
	     char dir;                  // Aktuelle Richtung
					// 0 ... links, 1 ... rechts

	     char status;               // 0 ... tot
					// 1 ... aktiv
					// 2 ... betubt / zum nehmen
					// 3 ... unsanft gewckt
					// 4 ... "warten"
					// 5 ... Beit zu
					// 6 ... Dreht um
					// 7 ... (Sturm aktiviert sich)
	     CObject();
	     CObject(int sx, int sy, int sdir);
	     ~CObject();

	     virtual void Move();
	     virtual void Draw(int page);
	     virtual void Kick(int kx, int ky, CPlayer *who);
	     virtual void Kicked(int pos);
	     virtual void GetPos(int *rx, int *ry)
			  {
			   *rx=x; *ry=y;
			  }
	     virtual void Collision();
	     virtual void Kill(int pos);
	     virtual void Kill(int pos, int type);
	     virtual void Turn();
	     virtual void Destroy();
	     virtual void SetMaxSpeed();
};

class CPlayer:public CObject
{
 private: char color;
	  char kicked;
	  char deathtype;
	  unsigned *keyb;
	  int g;
	  unsigned aindex;

 public: word stormtimer;
	 int nr;                             // # Player
	 CPlayer(unsigned nr,unsigned *keys);
	 ~CPlayer();

	 void Move();
	 void Draw(int page);
	 void Kick(int kx, int ky, CPlayer *who);
	 void Kicked(int pos);
	 void Collision();
	 void GetPos(int *rx, int *ry)
		    {
		     *rx=x>>3;
		     *ry=y;
		    }
	 void Kill(int dir, int type);
	 void Destroy();
	 void SetXY(int xx, int yy) {x=xx<<3; y=yy;};
};

class CTurtle:public CObject
{
 private: unsigned char colors[4];
	  word aanim;

 public: CTurtle(int nx, int ny,int nd,unsigned delay);
	 ~CTurtle();

	 void Move();
	 void Draw(int page);
	 void Kicked(int pos);
	 void GetPos(int *rx, int *ry)
		    {
		     *rx=x/3;
		     *ry=y;
		    }
	 void Kill(int dir);
	 void SetMaxSpeed() { speed = 3; }
	 void Turn();
};

class CRobot:public CObject
{
 private: int aindex;
	  int aanim;
	  int bkupindex;
	  unsigned char colors[4];


 public: CRobot(int nx, int ny, int nd,unsigned delay);

	 void Move();
	 void Draw(int page);
	 void Kicked(int pos);
	 void GetPos(int *rx, int *ry)
		    {
		     *rx=x/3;
		     *ry=y;
		    }
	 void Kill(int dir);
	 void SetMaxSpeed() { speed = 6 - (speed&1);}
	 void Turn();
};

class CHumer:public CObject
{
 private: int ftimer;
	  int aindex;

 public: CHumer(int nx, int ny, int nd,unsigned delay);

	 void Move();
	 void Draw(int page);
	 void Kicked(int pos);
	 void GetPos(int *rx, int *ry)
		    {
		     *rx=x/3;
		     *ry=y;
		    }
	 void Kill (int dir);
	 void Collision();
	 void SetMaxSpeed() { speed = 3; }
};

class CDrop:public CObject
{

 public: CDrop(int nx, int ny, int nd,unsigned delay);

	 void Move();
	 void Draw(int page);
	 void Kicked(int pos);
	 void GetPos(int *rx, int *ry)
		    {
		     *rx=x/3;
		     *ry=y;
		    }
	 void Collision();
	 void Destroy();
};

class CCoin:public CObject
{
 private: int animation[16];

 public: CCoin(int nx, int ny, int nd,unsigned delay);

	 void Move();
	 void Draw(int page);
	 void Kicked(int pos);
	 void GetPos(int *rx, int *ry)
		    {
		     *rx=x/3;
		     *ry=y;
		    }
	 void Kill(int dir);
	 void Collision();
	 void Destroy();
};

class CStorm: public CObject
{
 private: int atimer,y0;
	  int rval;

 public: CStorm (int nx, int ny, int nd);

	 void Move();
	 void Draw(int page);
	 void Kicked(int pos);
	 void GetPos(int *rx, int *ry)
		    {
		     *rx=x/3;
		     *ry=y;
		    }
	 void Collision();
	 void Destroy();
};


unsigned char VGetPixel(word x, word y);




class Statusbar
{
 private: unsigned screen;
	  void tonumber (char *string, long x);

 public:  long score[2];
	  unsigned lives[2];
	  long topscore;

	  Statusbar(unsigned scorf);
	  ~Statusbar();

	  void Update();
	  void UpdateScore(unsigned player);
	  void UpdateLives(unsigned player);
	  void UpdateTop();
};

class CPow:public CObject
{
 private: word size;
	  byte colors[16];
	  word colorpt;

 public: CPow();
	 ~CPow();

	 void Restore() {size=3; }
	 void Draw(int page);
	 void Kick(int kx, int ky, CPlayer *who);
	 void Kicked(int pos);
	 void Destroy();
	 void GetPos(int *rx, int *ry)  {*rx=1000; *ry=1000; }

};


/***************** Random selbst definiert **********************/
// Damit ein 386'er 32-Bit Code generiert werde kann


int myrand(void);
void mysrand(unsigned seed);


inline int _Cdecl myrandom(int __num)
		 { return(int)(((long)myrand()*__num)/(RAND_MAX+1)); }
/* need prototype of time() for C++ randomize() */
extern "C" long _Cdecl _FARFUNC time(long _FAR *);
inline void _Cdecl myrandomize(void) { mysrand((unsigned) time(NULL)); }
